import { resolve as pathResolve, dirname, join } from 'path'
import { access, accessSync, statSync } from 'fs'
import { execSync as exec } from 'child_process'
import findModules from 'find-node-modules'
import findUpSync from 'findup-sync'

export function findPackageSync (packageName, cwd = process.cwd(), searchFor = 'node_modules') {
  let moduleDirectories = findModules(cwd, { relative: false, searchFor })

  let directory = moduleDirectories.find((p) => {
    let exists = pathExists(join(p, packageName))
    return exists
  })

  if (!directory) {
    try {
      let resolvedPath = dirname(require.resolve(packageName))
      return resolvedPath
    } catch(error) {
      //console.log('Error looking up package', packageName, error.message)
    }
  }

  return directory && pathResolve( join(directory, packageName))
}

export function findPackage (packageName, searchFor = 'node_modules') {
  return new Promise((resolve, reject) => {
    let moduleDirectories = findModules(process.cwd(), {searchFor, relative:false})

    let directory = moduleDirectories.find((p) => {
      let exists = pathExists(join(p, packageName))
      return exists
    })

    if (!directory) {
      try {
        let resolvedPath = dirname(require.resolve(packageName))
        resolve(resolvedPath)
        return
      } catch(error) {

      }
    }

    if (!directory) { reject(packageName) }

    let result = pathResolve(
      join(directory, packageName)
    )

    if(result) { resolve(result) }
  })
}

export function splitPath(p = '') {
   return pathResolve(p).split(sep)
}

export function pathExists(fp) {
	var fn = typeof access === 'function' ? accessSync : statSync

  try {
     fn(fp)
     return true
  } catch(error) {
    return false
  }
}

export function findPackageManifest (cwd = process.cwd()) {
  return findUpSync('package.json', {cwd})
}

/**
 * Run nodes child_promise exec synchronously and capture the output, trimming any newlines or leading spaces
 * @param  {String} cmd     The command string, exactly the same as child_process.execSync
 * @param  {Object} options =             {} Any arguments for execSync will work here
 * @return {String}         Returns the command output
 */
export function execSync(cmd, options = {}) {
  return exec(
    cmd, options
  ).toString().trim()
}
