import * as util from './properties'

const { getOwnPropertyDescriptor } = Object

describe('Skypager Util', function() {
  it('hides a plain property', function() {
    const obj = { visible: true }
    util.hideProperty(obj, 'invisible', () => true, false)

    const descriptor = getOwnPropertyDescriptor(obj, 'invisible')

    descriptor.should.have.property('value').that.is.a('function')
    descriptor.should.have.property('enumerable').that.equals(false)
    descriptor.should.have.property('configurable').that.equals(false)
  })

  it('hides a getter', function() {
    const obj = { visible: true }
    util.hideGetter(obj, 'invisible', () => true, false)

    const descriptor = getOwnPropertyDescriptor(obj, 'invisible')

    descriptor.should.have.property('get').that.is.a('function')
    descriptor.should.have.property('enumerable').that.equals(false)
    descriptor.should.have.property('configurable').that.equals(false)
  })

  it('provides shortcuts for the property methods', function() {
    const target = {}
    const propertyUtils = util.propertyUtils(target)

    propertyUtils.should.have.property('hide')
    propertyUtils.should.have.property('lazy')
    propertyUtils.should.have.property('hideProperty')
    propertyUtils.should.have.property('hideProperties')
    propertyUtils.should.have.property('hideGetter')

    propertyUtils.hide('myStuff', 1)

    target.should.have.property('myStuff')
    getOwnPropertyDescriptor(target, 'myStuff').should
      .have.property('enumerable', false)
  })

  it('helps create lazy loading properties', function() {
    let untouched = false
    const target = {}
    util.lazy(target, 'touched', (() => untouched = true), true)

    untouched.should.equal(false)

    getOwnPropertyDescriptor(target, 'touched').should
      .have.property('configurable', true)

    getOwnPropertyDescriptor(target, 'touched').should
      .have.property('get')
      .that.is.a('function')

    getOwnPropertyDescriptor(target, 'touched').should
        .have.property('enumerable', true)

    getOwnPropertyDescriptor(target, 'touched').should
        .have.property('get')
        .that.is.a('function')

    untouched.should.equal(false)
    target.touched.should.equal(true)

    getOwnPropertyDescriptor(target, 'touched').should
        .have.property('value', true)

    getOwnPropertyDescriptor(target, 'touched').should
        .have.property('enumerable', true)
  })
})
