import partial from 'lodash/partial'
import isObject from 'lodash/isObject'
import { hideProperty, hideGetter, lazy } from '.'
const { assign, getOwnPropertyDescriptors, keys } = Object
const descriptors = getOwnPropertyDescriptors

/**
 * Takes a host object and attaches a registry object to it. The registry object is
 * an interface that allows you to query for information about its member functions
 * member functions are any function that you want to register by name. they will be memoized
 * lazy loading functions that can be called in the future.
 *
 */
export function create (host = module.exports, propKey = 'registry', members = {}) {
  if (!isObject(host)) {
    throw 'You must pass a host object'
  }

  const cacheKey = `_${propKey}`

  if (!host[cacheKey]) {
    hideProperty(host, cacheKey, {})
  }

  let getters = {}

  const available = keys(descriptors({
    ...host[cacheKey],
    ...members
  }))


  available.forEach(
    key => assign(getters, {get [key]() { return host[cacheKey][key] }})
  )

  hideGetter(host, propKey, () => {
    keys(descriptors(host[cacheKey])).forEach(
      key => assign(getters, {get [key]() { return host[cacheKey][key] }})
    )

    return {
      ...getters,
      get available() {
        return keys(descriptors(host[cacheKey]))
      },
      get register() {
        return partial(lazy, host[cacheKey])
      }
    }
  })

  return host[propKey]
}

export default create
